/*
 * Copyright (c) 2012 AgroSense.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    AgroSense - initial API and implementation and/or initial documentation
 */
package nl.cloudfarming.mojo.translate.omegat;

import java.io.File;
import java.io.IOException;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.shared.model.fileset.FileSet;
import org.apache.maven.shared.model.fileset.util.FileSetManager;
import org.codehaus.plexus.util.FileUtils;

/**
 * Goal which copies files to be translated to an OmegaT project.
 *
 * @goal copy-sources
 * @phase process-classes
 */
public class CopySourcesMojo extends AbstractTransferMojo {
    /**
     * Path to the shared sources directory, relative to the translation root directory.
     *
     * @parameter default-value="shared-sources"
     * @required
     */
    protected String sharedSourcesPath;    
    
    // maybe later: 
    // choose between a shared sources directory or individual copies per translation project (i.e. per language)
    // private Boolean shareSources; 

    @Override
    public void execute() throws MojoExecutionException {
        init();
        
        final FileSetManager fileSetManager = new FileSetManager();

        File translationSourcesRoot = new File(getTranslationRootDirectory(), sharedSourcesPath);
        getLog().info("Copy sources to: " + translationSourcesRoot.getAbsolutePath());
        
        // clean source directory via parent project before copying new/updated files from modules:
        if (project.isExecutionRoot()) {
            try {
                getLog().info("Cleaning translation sources directory");
                FileUtils.cleanDirectory(translationSourcesRoot);
            } catch (IOException ex) {
                throw new MojoExecutionException("error while cleaning translation sources", ex);
            }
        }
        
        File source, target;
                
        for (FileSet fileset : filesets) {
            getLog().info("Processing fileset for directory: " + fileset.getDirectory());
            String[] includedFiles = fileSetManager.getIncludedFiles(fileset);
            if (includedFiles.length == 0) {
                getLog().info("No files to copy");
            } else {
                getLog().info(includedFiles.length + " file(s) to copy:");
                for (int i = 0; i < includedFiles.length; i++) {
                    getLog().info("copy " + includedFiles[i]);
                    source = new File(fileset.getDirectory(), includedFiles[i]);
                    // TODO: we might need to add another directory level between the modules (artifactId) 
                    // and the source filepath to prevent conflicts when using multiple filesets.
                    // Not yet an issue since we'll use at most one of "target/classes" and "src/main/resources"
                    // which contain "nl/cloudfarming/*" packages, optionally combined with "src/main/nbm-branding"
                    // which contains "core" and "modules" subdirectories.
                    target = new File(translationSourcesRoot, project.getArtifactId() + File.separator + includedFiles[i]);
                    copyFile(source, target);
                }
            }
        }
    }
}
